/*
NaverDic Widget v3.5
Copyright (c) 2007 NeXTcube, by capri91
*/

/*
:: References ::
1) Unicode 5.0 - UAC00 (Hangul Syllables, Range: AC00-D7AF)
2) Unicode 5.0 - U1100 (Hangul Jamo, Range: 1100-11FF)
3) 김복문 로마자 표기법 (영어의 발음을 기준으로 만든 모의 발음 부호법)
4) 국립국어원 로마자 표기법, 국립국어원 표준어 규정
5) KSC 5601 (Character Set, Range: 0xB0A1-0xC8FE)
*/

var version = 3.5;
var check;
var today = new Date();
var current = today.getTime();
var defX = 410;
var defY = 305;
var shrinkX = 410;
var shrinkY = 60;
var thumbStartY;
var clickOffset;
var oldSize;
var isStretched = false;
var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};
var searchTerm;
var speaking = null;
var x = defX;
var y = defY;
var dic = "사전통합";
var speech = "Yes";
var speechKr = "Yes";
var voice = "Alex";
var debug = 0;

function load() {
	var versionStr = widget.preferenceForKey("version");
	var checkStr = widget.preferenceForKey("check");
	var xStr = widget.preferenceForKey("x");
	var yStr = widget.preferenceForKey("y");
	var dicStr = widget.preferenceForKey("dic");
	var speechStr = widget.preferenceForKey("speech");
	var speechKrStr = widget.preferenceForKey("speechKr");
	var voiceStr = widget.preferenceForKey("voice");
	var debugStr = widget.preferenceForKey("debug");

	if (versionStr) {
		if (versionStr > version)
			document.getElementById("newLogo").style.display = "block";
	}

	if (checkStr) {
		check = checkStr;
	}
	else {
		check = current;

		if (window.widget)
			widget.setPreferenceForKey(current,"check");
	}

	if (xStr && yStr) {
		x = xStr;
		y = yStr;

		window.resizeTo(shrinkX, shrinkY);
	}

	if (dicStr) {
		dic = dicStr;

		document.getElementById("dicText").innerText = dicStr;
		document.getElementById("dicOption").value = dicStr;
	}

	if (speechStr) {
		speech = speechStr;

		document.getElementById("speechOption").value = speechStr;
	}

	if (speechKrStr) {
		speechKr = speechKrStr;

		document.getElementById("speechKrOption").value = speechKrStr;
	}

	if (voiceStr) {
		voice = voiceStr;

		document.getElementById("voiceOption").value = voiceStr;
	}

	if (debugStr) {
		debug = debugStr;

		if (debug == 1)
			document.getElementById("debugOption").checked = true;
		else
			document.getElementById("debugOption").checked = false;
	}

	document.getElementById("content").innerHTML = "";

	showAll("first");
	closeAll("first");
	checkVersion();

	document.getElementById("resize").style.display = "block";
}

function showBack() {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToBack");

	front.style.display = "none";
	back.style.display = "block";

	restoreFocus();

	if (window.widget)
		setTimeout("widget.performTransition();", 0);

	document.getElementById("fliprollie").style.display = "none";

	doneSpeak();
}

function hideBack() {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToFront");

	front.style.display = "block";
	back.style.display = "none";

	if (window.widget)
		setTimeout("widget.performTransition();", 0);

	restoreFocus();
}

function mousemove(event) {
	if (!flipShown) {
		if (animation.timer != null) {
			clearInterval(animation.timer);
			animation.timer = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById("flip");
		animation.timer = setInterval("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();

		flipShown = true;
	}
}

function mouseexit(event) {
	if (flipShown) {
		if (animation.timer != null) {
			clearInterval(animation.timer);
			animation.timer = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById("flip");
		animation.timer = setInterval("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();

		flipShown = false;
	}
}

function mouseDown(event) {
	oldSize = {x:window.innerWidth, y:window.innerHeight};

	var diffX = oldSize.x - event.x;
	var diffY = oldSize.y - event.y;

	clickOffset = {x:diffX, y:diffY};

	document.addEventListener("mousemove", mouseMove, true);
	document.addEventListener("mouseup", mouseUp, true);

	event.stopPropagation();
	event.preventDefault();
}

function mouseMove(event) {
	newSize = {x:(clickOffset.x + event.x), y:(clickOffset.y + event.y)};

	if (newSize.x < defX) {
		newSize.x = defX;
	}

	if (newSize.y < defY) {
		newSize.y = defY;
	}

	if (oldSize.x != newSize.x || oldSize.y != newSize.y) {
		x = newSize.x;
		y = newSize.y;

		window.resizeTo(newSize.x, newSize.y);

		thumbStartY = -1;

		calculateAndShowThumb(document.getElementById("content"));
	}

	event.stopPropagation();
	event.preventDefault();
}

function mouseUp(event) {
	document.removeEventListener("mousemove", mouseMove, true);
	document.removeEventListener("mouseup", mouseUp, true); 

	widget.setPreferenceForKey(x,"x");
	widget.setPreferenceForKey(y,"y");

	event.stopPropagation();
	event.preventDefault();
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();

	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration) {
		clearInterval(animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		// for color syntax /
		animation.now = computeNextFloat(animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
}

function limit_3(a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
	return from + (to - from) * ease;
}

function enterflip(event) {
	document.getElementById("fliprollie").style.display = "block";
}

function exitflip(event) {
	document.getElementById("fliprollie").style.display = "none";
}

function showAll(onload) {
	document.getElementById("frontBox").style.display = "block";

	var i = shrinkX;
	var k = (x-shrinkX) / 20;
	var l = (y-shrinkY) / 20;

	if (!isStretched) {
		if (onload == "first") {
			for (var j = shrinkY; j < y; j = j+l) {
				window.resizeTo(shrinkX, j);
			}
			window.resizeTo(shrinkX, y);
		}
		else {
			for (var j = shrinkY; j < y; j = j+l) {
				if (i < x)
					i = i+k;
				window.resizeTo(i, j);
			}
			window.resizeTo(x, y);
		}
	}

	thumbStartY = -1;

	calculateAndShowThumb(document.getElementById("content"));

	isStretched = true;

	restoreFocus();
}

function closeAll(onload) {
	var i = x;
	var k = (x-shrinkX) / 20;
	var l = (y-shrinkY) / 20;

	if (isStretched) {
		if (onload == "first") {
			for (var j = y; j > shrinkY; j = j-l) {
				window.resizeTo(shrinkX, j);
			}
			window.resizeTo(shrinkX, shrinkY);
		}
		else {
			for (var j = y; j > shrinkY; j = j-l) {
				if (i > shrinkX)
					i = i-k;
				window.resizeTo(i, j);
			}
			window.resizeTo(shrinkX, shrinkY);
		}

		thumbStartY = -1;

		calculateAndShowThumb(document.getElementById("content"));

		document.getElementById("frontBox").style.display = "none";

		isStretched = false;

		doneSpeak();
	}
}

function checkVersion() {
	var timeDiff = current - check;
	var versionUrl = "http://www.nextcube.org/version/NaverDic.txt";

	if (timeDiff == 0 || timeDiff > 2592000000) {
		versionReq = new XMLHttpRequest();
		versionReq.onreadystatechange = checkingVersion;
		versionReq.open("GET", versionUrl, true);
		versionReq.send(null);
	}
}

function checkingVersion() {
	if (versionReq.readyState == 4 && versionReq.status == 200) {
		if (versionReq.responseText > version) {
			document.getElementById("newLogo").style.display = "block";

			if (window.widget)
				widget.setPreferenceForKey(versionReq.responseText,"version");
		}
		else {
			document.getElementById("newLogo").style.display = "none";

			if (window.widget)
				widget.setPreferenceForKey(version,"version");
		}

		if (window.widget)
			widget.setPreferenceForKey(current,"check");
	}
	else if (versionReq.readyState == 4) {
		versionReq.abort();
		versionReq = null;
	}
}

function keyPressed(evt) {
	var keyCode = evt.keyCode;

	if (27 == keyCode)
		closeAll();
}

function restoreFocus() {
	var searchfield = document.getElementById("searchfield");

	if (searchfield != null) {
		searchfield.focus();
		searchfield.select();
	}
}

function openLinkInBrowser(url) {
	if (window.widget) {
		widget.system("open '"+url+"'", null);
		widget.openURL("");
	}
}

function selectDic(changeDic, searchTerm) {
	dic = changeDic;

	widget.setPreferenceForKey(dic,"dic");

	document.getElementById("dicText").innerText = changeDic;

	search(searchTerm);
}

function search(searchTerm) {
	if (searchTerm != "") {
		searchTerm = convertSC(searchTerm);

		var response = "<br>검색어 '"+searchTerm+"'에 대해 검색 중입니다.";

		document.getElementById("content").innerHTML = response;

		showAll();

		window.resizeBy(0, 1);
		window.resizeBy(0, -1);

		thumbStartY = -1;

		calculateAndShowThumb(document.getElementById("content"));

		if (searchTerm == " ") {
			var response = "<br>검색어가 정확한지 확인해 주세요.";

			document.getElementById("content").innerHTML = response;

			showAll();
		}
		else {
			searching(searchTerm);
		}
	}
	else {
		closeAll();
	}
}

function searching(searchTerm) {
	if (dic == "사전통합") {
		var replaceTerm = searchTerm.replace(/ /g, "+");
		var url = "http://dic.naver.com/search.nhn?query="+replaceTerm;
	}
	else if (dic == "영어사전") {
		var url = "http://endic2009.naver.com/small_search.nhn?kind=keyword&query="+searchTerm;
	}
	else if (dic == "영영사전") {
		var url = "http://eedic2009.naver.com/small.naver?where=keyword&query="+searchTerm;
	}
	else if (dic == "국어사전") {
		var url = "http://krdic.naver.com/small_search.nhn?query_euckr="+searchTerm;
	}
	else if (dic == "한자사전") {
		var url = "http://hanja.naver.com/small/search?query="+searchTerm;
	}
	else if (dic == "일어사전") {
		var url = "http://jpdic.naver.com/mini_search.nhn?query="+searchTerm;
	}
	else if (dic == "중어사전") {
		var url = "http://cndic.naver.com/mini/search/all?direct=false&q="+searchTerm;
	}
	else if (dic == "백과사전") {
		var url = "http://100.naver.com/search.nhn?query="+searchTerm;
	}

	req = new XMLHttpRequest();
	req.open("GET", url ,false);
	req.send(null);

	if (req.readyState == 4) {
		var res = req.responseText;

		if (dic == "사전통합") {
			var resultsZero = res.lastIndexOf("<!--  no result -->");
			var resultsStart = res.indexOf("<!-- body -->");
			var resultsEnd = res.lastIndexOf("<div class=\"aside\">");
		}
		else if (dic == "영어사전") {
			var resultsZero = res.lastIndexOf("<!-- 0건 안내 -->");
			var resultsStart = res.indexOf("<!-- 컨텐츠 영역-->");
			var resultsEnd = res.lastIndexOf("<!-- //컨텐츠 영역-->");
		}
		else if (dic == "영영사전") {
			var resultsZero = res.lastIndexOf("<!-- 0건 안내 -->");
			var resultsStart = res.indexOf("<!-- 컨텐츠 영역-->");
			var resultsEnd = res.lastIndexOf("<!-- //컨텐츠 영역-->");
		}
		else if (dic == "국어사전") {
			var resultsZero = res.lastIndexOf("<!-- 0건 안내 -->");
			var resultsStart = res.indexOf("<!-- 내용-->");
			var resultsEnd = res.lastIndexOf("<!-- //내용-->");
		}
		else if (dic == "한자사전") {
			var resultsZero = res.lastIndexOf("검색결과가 없습니다");
			var resultsStart = res.indexOf("<!-- //검색결과 -->");
			var resultsEnd = res.lastIndexOf("<div class=\"totop\">");
		}
		else if (dic == "일어사전") {
			var resultsZero = res.lastIndexOf("검색결과가 없습니다");
			var resultsStart = res.indexOf("<!-- //검색결과 -->");
			var resultsEnd = res.lastIndexOf("<div class=\"top\">");
		}
		else if (dic == "중어사전") {
			var resultsZero = res.lastIndexOf("검색결과가 없습니다");
			var resultsStart = res.indexOf("<!-- content -->");
			var resultsEnd = res.lastIndexOf("<!-- //content -->");
		}
		else if (dic == "백과사전") {
			var resultsZero = res.lastIndexOf("검색 결과가 없습니다");
			var resultsStart = res.indexOf("<!-- 가운데 영역 -->");
			var resultsEnd = res.lastIndexOf("<!-- //가운데 영역 -->");
		}

		if (resultsZero > -1) {
			var response = "<br>검색어 '"+searchTerm+"'에 대한 검색결과가 없습니다.";
		}
		else if (resultsEnd > -1) {
			res = res.substring(resultsEnd, resultsStart);

			if (NaverDic)
				var response = NaverDic.getRES(res, dic);
			else
				alert("Widgetplugin not loaded.");

			if (speech == "Yes" && speechKr == "Yes") {
				if (checkASCII(searchTerm))
					convertASCII(searchTerm);
				else if (checkALL(searchTerm))
					convertUTF8(searchTerm);
			}
			else {
				if (speech == "Yes") {
					if (checkASCII(searchTerm))
						convertASCII(searchTerm);
				}
				else if (speechKr == "Yes") {
					if (checkUTF8(searchTerm))
						convertUTF8(searchTerm);
				}
			}
		}
		else {
			var response = "<br>검색어가 정확한지 확인해 주세요.";
		}

		document.getElementById("content").innerHTML = response;

		showAll();
	}
}

function selectSpeech(changeSpeech) {
	speech = changeSpeech;

	if (window.widget)
		widget.setPreferenceForKey(speech,"speech");
}

function selectSpeechKr(changeSpeech) {
	speechKr = changeSpeech;

	if (window.widget)
		widget.setPreferenceForKey(speechKr,"speechKr");
}

function selectVoice(changeVoice) {
	voice = changeVoice;

	if (window.widget)
		widget.setPreferenceForKey(voice,"voice");
}

function selectDebug() {
	if (document.getElementById("debugOption").checked)
		debug = 1;
	else
		debug = 0;

	if (window.widget)
		widget.setPreferenceForKey(debug,"debug");
}

function speak(speakTerm) {
	if (window.widget) {
		if (speaking != null)
			doneSpeak();

		speaking = widget.system("/usr/bin/osascript -e 'say \"" + speakTerm + "\" using \"" + voice + "\"'", doneSpeak);
	}
}

function doneSpeak() {
	if (speaking != null)
		speaking.cancel();

	speaking = null;
}

function checkSC(searchTerm) {
	for (var i = 0; i < searchTerm.length; i++) {
		var c = searchTerm.charCodeAt(i);

		if ((0x21 <= c && c <= 0x2F) || (0x3A <= c && c <= 0x40) || (0x5B <= c && c <= 0x60) || (0x7B <= c && c <= 0x7E))
			return true;
	}

	return false;
}

function checkASCII(searchTerm) {
	for (var i = 0; i < searchTerm.length; i++) {
		var c = searchTerm.charCodeAt(i);

		if (!(0x20 <= c && c <= 0x7A))
			return false;
	}

	return true;
}

function checkUTF8(searchTerm) {
	for (var i = 0; i < searchTerm.length; i++) {
		var c = searchTerm.charCodeAt(i);

		if (!((0x20 <= c && c <= 0x40) || (0x3131 <= c && c <= 0x314e) || (0xAC00 <= c && c <= 0xD7A3)))
			return false;
	}

	return true;
}

function checkALL(searchTerm) {
	for (var i = 0; i < searchTerm.length; i++) {
		var c = searchTerm.charCodeAt(i);

		if (!((0x20 <= c && c <= 0x7A) || (0x3131 <= c && c <= 0x314e) || (0xAC00 <= c && c <= 0xD7A3)))
			return false;
	}

	return true;
}

function convertSC(searchTerm) {
	if (checkSC(searchTerm)) {
		searchTerm = searchTerm.replace(/\!/g, " ");
		searchTerm = searchTerm.replace(/\"/g, " ");
		searchTerm = searchTerm.replace(/\#/g, " ");
		searchTerm = searchTerm.replace(/\$/g, " ");
		searchTerm = searchTerm.replace(/\%/g, " ");
		searchTerm = searchTerm.replace(/\&/g, " ");
		searchTerm = searchTerm.replace(/\'/g, " ");
		searchTerm = searchTerm.replace(/\(/g, " ");
		searchTerm = searchTerm.replace(/\)/g, " ");
		searchTerm = searchTerm.replace(/\*/g, " ");
		searchTerm = searchTerm.replace(/\+/g, " ");
		searchTerm = searchTerm.replace(/\,/g, " ");
		searchTerm = searchTerm.replace(/\-/g, " ");
		searchTerm = searchTerm.replace(/\./g, " ");
		searchTerm = searchTerm.replace(/\//g, " ");
		searchTerm = searchTerm.replace(/\:/g, " ");
		searchTerm = searchTerm.replace(/\;/g, " ");
		searchTerm = searchTerm.replace(/\</g, " ");
		searchTerm = searchTerm.replace(/\=/g, " ");
		searchTerm = searchTerm.replace(/\>/g, " ");
		searchTerm = searchTerm.replace(/\?/g, " ");
		searchTerm = searchTerm.replace(/\@/g, " ");
		searchTerm = searchTerm.replace(/\[/g, " ");
		searchTerm = searchTerm.replace(/\\/g, " ");
		searchTerm = searchTerm.replace(/\]/g, " ");
		searchTerm = searchTerm.replace(/\^/g, " ");
		searchTerm = searchTerm.replace(/\_/g, " ");
		searchTerm = searchTerm.replace(/\`/g, " ");
		searchTerm = searchTerm.replace(/\{/g, " ");
		searchTerm = searchTerm.replace(/\|/g, " ");
		searchTerm = searchTerm.replace(/\}/g, " ");
		searchTerm = searchTerm.replace(/\~/g, " ");
	}

	searchTerm = searchTerm.replace(/[ ]{1,}/g, " ");
	searchTerm = searchTerm.replace(/^ /g, "");
	searchTerm = searchTerm.replace(/ $/g, "");

	if (searchTerm == "")
		searchTerm = " ";

	document.getElementById("searchfield").value = searchTerm;

	return searchTerm;
}

function convertASCII(searchTerm) {
	var speakTerm = searchTerm;

	speak(speakTerm);
}

function convertUTF8(searchTerm)
{
	var speakTerm = encodeURIComponent(widget.system("echo '" + searchTerm + "' | iconv -c -s -f utf8 -t utf8", null).outputString);

	if (NaverDic)
		var speakTerm = NaverDic.getUTF8(speakTerm);
	else
		alert("Widgetplugin not loaded.");

	speak(speakTerm);
}
